﻿using System.Drawing;
using System.Drawing.Imaging;

namespace TestProject
{
    /// <summary>
    /// 이미지 확장
    /// </summary>
    public static class ImageExtension
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region RGB 감마 조정하기 - AdjustRGBGamma(sourceImage, red, green, blue, gamma)

        /// <summary>
        /// RGB 감마 조정하기
        /// </summary>
        /// <param name="sourceImage">소스 이미지</param>
        /// <param name="red">빨강색</param>
        /// <param name="green">녹색</param>
        /// <param name="blue">파랑색</param>
        /// <param name="gamma">감마</param>
        /// <returns>RGB 감마 조정 이미지</returns>
        /// <remarks>
        /// blue (metro) : 0f, 1f, 3f, 0.55f
        /// light grey   : 1f, 1f, 1f, 0.35f
        /// white        : 1f, 1f, 1f, 0.01f
        /// green        : 0f, 2f, 0f, 0.75f
        /// red          : 2f, 0f, 0f, 0.75f
        /// </remarks>
        public static Image AdjustRGBGamma(this Image sourceImage, float red, float green, float blue, float gamma)
        {
            Bitmap targetBitmap = new Bitmap(sourceImage.Width, sourceImage.Height);

            ImageAttributes imgAttributes = new ImageAttributes();

            ColorMatrix colorMatrix = new ColorMatrix
            (
                new float[][]
                {
                    new float[] { red, 0f   , 0f  , 0f, 0f },
                    new float[] { 0f , green, 0f  , 0f, 0f },
                    new float[] { 0f , 0f   , blue, 0f, 0f },
                    new float[] { 0f , 0f   , 0f  , 1f, 0f },
                    new float[] { 0f , 0f   , 0f  , 0f, 1f },
                }
            );

            imgAttributes.ClearColorMatrix();

            imgAttributes.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);

            imgAttributes.SetGamma(gamma, ColorAdjustType.Bitmap);

            using(Graphics graphics = Graphics.FromImage(targetBitmap))
            {
                graphics.DrawImage
                (
                    sourceImage,
                    new Rectangle(0, 0, sourceImage.Width, sourceImage.Height),
                    0,
                    0,
                    sourceImage.Width,
                    sourceImage.Height,
                    GraphicsUnit.Pixel,
                    imgAttributes
                );
            }

            return (Image)targetBitmap;
        }

        #endregion
    }
}