﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            if(MetroUI.IsDesignMode == false)
            {
                MetroUI.Style.PropertyChanged += Style_PropertyChanged;

                MetroUI.Style.IsDarkStyle = true;
            }

            this.exitApplicationMenuItem.Click += exitApplicationMenuItem_Click;
            this.switchStyleButton.Click       += switchStyleButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 윈도우 메시지 처리하기 - WndProc(message)

        /// <summary>
        /// 윈도우 메시지 처리하기
        /// </summary>
        /// <param name="message">메시지</param>
        protected override void WndProc(ref Message message)
        {
            if(message.Msg == 0x84) // WM_NCHITTEST
            {
                message.Result = (IntPtr)0x02;  // HTCAPTION
            }
            else
            {
                base.WndProc(ref message);
            }
        }

        #endregion
        #region 크기 조정시 처리하기 - OnResize(e)

        /// <summary>
        /// 크기 조정시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            Invalidate();
        }

        #endregion
        #region 페인트시 처리하기 - OnPaint(e)

        /// <summary>
        /// 페인트시 처리하기
        /// </summary>
        /// <param name="e">이벤트인자</param>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            using(Pen pen = new Pen(MetroUI.Style.AccentColor))
            {
                e.Graphics.DrawRectangle(pen, 0, 0, Width - 1, Height - 1);
            }

            using(SolidBrush brush = new SolidBrush(MetroUI.Style.AccentColor))
            {
                e.Graphics.DrawString(Text.ToUpper(), MetroUI.Style.LightFont, brush, 10, 8);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 스타일 속성 변경시 처리하기 - Style_PropertyChanged(sender, e)

        /// <summary>
        /// 스타일 속성 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Style_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if(e.PropertyName == "IsDarkStyle")
            {
                BackColor = MetroUI.Style.BackColor;

                Refresh();
            }
        }

        #endregion
        #region Exit Application 메뉴 항목 클릭시 처리하기 - exitApplicationMenuItem_Click(sender, e)

        /// <summary>
        /// Exit Application 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void exitApplicationMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion
        #region Switch Style 버튼 클릭시 처리하기 - switchStyleButton_Click(sender, e)

        /// <summary>
        /// Switch Style 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void switchStyleButton_Click(object sender, EventArgs e)
        {
            MetroUI.Style.IsDarkStyle = !(MetroUI.Style.IsDarkStyle);
        }

        #endregion
    }
}