﻿using System.ComponentModel;
using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 메트로 스타일
    /// </summary>
    public class MetroStyle : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 베이스 폰트
        /// </summary>
        private Font baseFont;

        /// <summary>
        /// 볼드체 폰트
        /// </summary>
        private Font boldFont;

        /// <summary>
        /// 라이트 폰트
        /// </summary>
        private Font lightFont;

        /// <summary>
        /// 배경색
        /// </summary>
        private Color backColor;

        /// <summary>
        /// 전경색
        /// </summary>
        private Color foreColor;

        /// <summary>
        /// 강조색
        /// </summary>
        private Color accentColor;

        /// <summary>
        /// 강조 전경색
        /// </summary>
        private Color accentForeColor;

        /// <summary>
        /// 비활성화 색
        /// </summary>
        private Color disabledColor;

        /// <summary>
        /// 다크 스타일 여부
        /// </summary>
        private bool isDarkStyle;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 베이스 폰트 - BaseFont

        /// <summary>
        /// 베이스 폰트
        /// </summary>
        public Font BaseFont
        {
            get
            {
                return this.baseFont;
            }
            set
            {
                if(value.Equals(this.baseFont) == false)
                {
                    this.baseFont = value;

                    FirePropertyChanged("BaseFont");
                }
            }
        }

        #endregion
        #region 볼드체 폰트 - BoldFont

        /// <summary>
        /// 볼드체 폰트
        /// </summary>
        public Font BoldFont
        {
            get
            {
                return this.boldFont;
            }
            set
            {
                if(value.Equals(this.boldFont) == false)
                {
                    this.boldFont = value;

                    FirePropertyChanged("BoldFont");
                }
            }
        }

        #endregion
        #region 라이트 폰트 - LightFont

        /// <summary>
        /// 라이트 폰트
        /// </summary>
        public Font LightFont
        {
            get
            {
                return this.lightFont;
            }
            set
            {
                if(value.Equals(this.lightFont) == false)
                {
                    this.lightFont = value;

                    FirePropertyChanged("LightFont");
                }
            }
        }

        #endregion
        #region 배경색 - BackColor

        /// <summary>
        /// 배경색
        /// </summary>
        public Color BackColor
        {
            get
            {
                return this.backColor;
            }
            set
            {
                if(value.Equals(this.backColor) == false)
                {
                    this.backColor = value;

                    FirePropertyChanged("BackColor");
                }
            }
        }

        #endregion
        #region 전경색 - ForeColor

        /// <summary>
        /// 전경색
        /// </summary>
        public Color ForeColor
        {
            get
            {
                return this.foreColor;
            }
            set
            {
                if(value.Equals(this.foreColor) == false)
                {
                    this.foreColor = value;

                    FirePropertyChanged("ForeColor");
                }
            }
        }

        #endregion
        #region 강조색 - AccentColor

        /// <summary>
        /// 강조색
        /// </summary>
        public Color AccentColor
        {
            get
            {
                return this.accentColor;
            }
            set
            {
                if(value.Equals(this.accentColor) == false)
                {
                    this.accentColor = value;

                    FirePropertyChanged("AccentColor");
                }
            }
        }

        #endregion
        #region 강조 전경색 - AccentForeColor

        /// <summary>
        /// 강조 전경색
        /// </summary>
        public Color AccentForeColor
        {
            get
            {
                return this.accentForeColor;
            }
            set
            {
                if(value.Equals(this.accentForeColor) == false)
                {
                    this.accentForeColor = value;

                    FirePropertyChanged("AccentForeColor");
                }
            }
        }

        #endregion
        #region 비활성화 색 - DisabledColor

        /// <summary>
        /// 비활성화 색
        /// </summary>
        public Color DisabledColor
        {
            get
            {
                return this.disabledColor;
            }
            set
            {
                if(value.Equals(this.disabledColor) == false)
                {
                    this.disabledColor = value;

                    FirePropertyChanged("DisabledColor");
                }
            }
        }

        #endregion
        #region 다크 스타일 여부 - IsDarkStyle

        /// <summary>
        /// 다크 스타일 여부
        /// </summary>
        public bool IsDarkStyle
        {
            get
            {
                return this.isDarkStyle;
            }
            set
            {
                if(value != this.isDarkStyle)
                {
                    this.isDarkStyle = value;

                    if(this.isDarkStyle)
                    {
                        BackColor       = Color.FromArgb(51, 51, 51);
                        ForeColor       = Color.White;
                        AccentColor     = Color.DarkOrange;
                        AccentForeColor = Color.FromArgb(51, 51, 51);
                        DisabledColor   = Color.DimGray;
                    }
                    else
                    {
                        BackColor       = Color.White;
                        ForeColor       = Color.FromArgb(51, 51, 51);
                        AccentColor     = Color.FromArgb(0, 114, 198);
                        AccentForeColor = Color.White;
                        DisabledColor   = Color.DimGray;
                    }

                    FirePropertyChanged("IsDarkStyle");
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MetroStyle()

        /// <summary>
        /// 생성자
        /// </summary>
        public MetroStyle()
        {
            BaseFont  = new Font("Segoe UI"      , 11.25f                   );
            BoldFont  = new Font("Segoe UI"      , 11.25f, FontStyle.Bold   );
            LightFont = new Font("Segoe UI Light", 11.25f, FontStyle.Regular);

            IsDarkStyle = false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChanged(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">property name</param>
        public void FirePropertyChanged(string propertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }
}